/*  -*- buffer-read-only: t -*- vi: set ro:
 *
 * DO NOT EDIT THIS FILE   (ag-text.h)
 *
 * It has been AutoGen-ed
 * From the definitions    /u/bkorb/tools/ag/autogen-bld/agen5/ag-text.def
 * and the template file   strings
 *
 *  Copyright (C) 2011-2018 Bruce Korb, all rights reserved.
 *  This is free software. It is licensed for use, modification and
 *  redistribution under the terms of the
 *  Modified (3 clause) Berkeley Software Distribution License
 *  <http://www.xfree86.org/3.3.6/COPYRIGHT2.html>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name ``Bruce Korb'' nor the name of any other
 *     contributor may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  strings IS PROVIDED BY Bruce Korb ``AS IS'' AND ANY EXPRESS
 *  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL Bruce Korb OR ANY OTHER CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#ifndef STRINGS_AG_TEXT_H_GUARD
#define STRINGS_AG_TEXT_H_GUARD 1
/*
 * 408 strings in ag_text_strtable string table
 */
#define ADDR_FAMILY_BAD              (ag_text_strtable+10)
#define ADDR_FAMILY_BAD_LEN          41
#define ADD_ENV_VARS_SUFFIX_FMT      (ag_text_strtable+52)
#define ADD_ENV_VARS_SUFFIX_FMT_LEN  4
#define ADD_SYS_ENV_VAL_FMT          (ag_text_strtable+57)
#define ADD_SYS_ENV_VAL_FMT_LEN      4
#define AG_ABEND_STATE_FMT           (ag_text_strtable+62)
#define AG_ABEND_STATE_FMT_LEN       22
#define AG_ABORT_LOC_FMT             (ag_text_strtable+85)
#define AG_ABORT_LOC_FMT_LEN         74
#define AG_FPRINTF_BAD_PORT          (ag_text_strtable+160)
#define AG_FPRINTF_BAD_PORT_LEN      29
#define AG_MAKE_DEP_NAME             (ag_text_strtable+190)
#define AG_MAKE_DEP_NAME_LEN         16
#define AG_MARK                      (ag_text_strtable+207)
#define AG_MARK_LEN                  8
#define AG_NEVER_STARTED             (ag_text_strtable+216)
#define AG_NEVER_STARTED_LEN         62
#define AG_SIG_ABORT_FMT             (ag_text_strtable+279)
#define AG_SIG_ABORT_FMT_LEN         47
#define AG_TEXT_STRTABLE_FILE        (ag_text_strtable+0)
#define AG_TEXT_STRTABLE_FILE_LEN    9
#define ALIST_TO_AG_ERR              (ag_text_strtable+327)
#define ALIST_TO_AG_ERR_LEN          52
#define ALIST_TO_AG_TEXT             (ag_text_strtable+380)
#define ALIST_TO_AG_TEXT_LEN         27
#define ALIST_TO_AG_WRAP             (ag_text_strtable+408)
#define ALIST_TO_AG_WRAP_LEN         23
#define APRF_ALLOCATE_FAIL           (ag_text_strtable+432)
#define APRF_ALLOCATE_FAIL_LEN       50
#define ASPRINTFV_FAIL_FMT           (ag_text_strtable+483)
#define ASPRINTFV_FAIL_FMT_LEN       26
#define ASSEMBLE_NAME_ERR            (ag_text_strtable+510)
#define ASSEMBLE_NAME_ERR_LEN        51
#define BAD_BREAK_FMT                (ag_text_strtable+562)
#define BAD_BREAK_FMT_LEN            29
#define BAD_MAC_NM_FMT               (ag_text_strtable+592)
#define BAD_MAC_NM_FMT_LEN           34
#define BAD_MSG_STR                  (ag_text_strtable+627)
#define BAD_MSG_STR_LEN              36
#define BAD_NAME_FMT                 (ag_text_strtable+664)
#define BAD_NAME_FMT_LEN             35
#define BAD_RE_FMT                   (ag_text_strtable+700)
#define BAD_RE_FMT_LEN               46
#define BASE_NAME_TOO_LONG           (ag_text_strtable+747)
#define BASE_NAME_TOO_LONG_LEN       28
#define BOGUS_TAG                    (ag_text_strtable+776)
#define BOGUS_TAG_LEN                11
#define BREAK_STR                    (ag_text_strtable+788)
#define BREAK_STR_LEN                5
#define CANNOT_FMT                   (ag_text_strtable+798)
#define CANNOT_FMT_LEN               28
#define CFGDEP_DUP_TARGET_MSG        (ag_text_strtable+827)
#define CFGDEP_DUP_TARGET_MSG_LEN    21
#define CFGDEP_UNKNOWN_DEP_FMT       (ag_text_strtable+849)
#define CFGDEP_UNKNOWN_DEP_FMT_LEN   28
#define CGI_ERR_MSG_FMT              (ag_text_strtable+878)
#define CGI_ERR_MSG_FMT_LEN          57
#define CGI_PARSE_ERR_FMT            (ag_text_strtable+936)
#define CGI_PARSE_ERR_FMT_LEN        22
#define CGI_TEMP_ERR_FILE_STR        (ag_text_strtable+959)
#define CGI_TEMP_ERR_FILE_STR_LEN    22
#define CMD_FAIL_FMT                 (ag_text_strtable+982)
#define CMD_FAIL_FMT_LEN             44
#define COMPILE_RE_BAD               (ag_text_strtable+1101)
#define COMPILE_RE_BAD_LEN           22
#define CONTINUE_STR                 (ag_text_strtable+1124)
#define CONTINUE_STR_LEN             8
#define C_STRING_NEWLINE             (ag_text_strtable+1133)
#define C_STRING_NEWLINE_LEN         12
#define DEF_ERR_FMT                  (ag_text_strtable+1027)
#define DEF_ERR_FMT_LEN              31
#define DEF_FILE_LINE_FMT            (ag_text_strtable+1146)
#define DEF_FILE_LINE_FMT_LEN        15
#define DEF_NOTE_FMT                 (ag_text_strtable+1059)
#define DEF_NOTE_FMT_LEN             41
#define DEF_TYPE_BLOCK               (ag_text_strtable+1162)
#define DEF_TYPE_BLOCK_LEN           5
#define DEF_TYPE_INVALID             (ag_text_strtable+1168)
#define DEF_TYPE_INVALID_LEN         7
#define DEF_TYPE_TEXT                (ag_text_strtable+1176)
#define DEF_TYPE_TEXT_LEN            4
#define DEF_TYPE_UNKNOWN             (ag_text_strtable+1181)
#define DEF_TYPE_UNKNOWN_LEN         7
#define DEP_FILE_CLEAN_FMT           (ag_text_strtable+1193)
#define DEP_FILE_CLEAN_FMT_LEN       91
#define DEP_FILE_SPLICE_STR          (ag_text_strtable+1189)
#define DEP_FILE_SPLICE_STR_LEN      3
#define DEP_FILE_WRAP_FMT            (ag_text_strtable+1285)
#define DEP_FILE_WRAP_FMT_LEN        48
#define DEP_List                     (ag_text_strtable+1334)
#define DEP_List_LEN                 6
#define DEP_OUT_NAME                 (ag_text_strtable+1341)
#define DEP_OUT_NAME_LEN             19
#define DEP_SList                    (ag_text_strtable+1361)
#define DEP_SList_LEN                12
#define DEP_TList                    (ag_text_strtable+1374)
#define DEP_TList_LEN                10
#define DEV_NULL                     (ag_text_strtable+1385)
#define DEV_NULL_LEN                 9
#define DFT_BASE_NAME                (ag_text_strtable+1395)
#define DFT_BASE_NAME_LEN            8
#define DFT_DEF_INPUT_STR            (ag_text_strtable+1404)
#define DFT_DEF_INPUT_STR_LEN        1
#define DFT_ENV_VAL                  (ag_text_strtable+1406)
#define DFT_ENV_VAL_LEN              2
#define DFT_TPL_DIR_DATA             (ag_text_strtable+1409)
#define DFT_TPL_DIR_DATA_LEN         2
#define DFT_TPL_DIR_RELATIVE         (ag_text_strtable+1412)
#define DFT_TPL_DIR_RELATIVE_LEN     19
#define DIRECT_ASSERT_FMT            (ag_text_strtable+1435)
#define DIRECT_ASSERT_FMT_LEN        27
#define DIRECT_BAD_CTX_FMT           (ag_text_strtable+1463)
#define DIRECT_BAD_CTX_FMT_LEN       61
#define DIRECT_CK_LIST_MARK          (ag_text_strtable+1432)
#define DIRECT_CK_LIST_MARK_LEN      2
#define DIRECT_ERROR_FMT             (ag_text_strtable+1525)
#define DIRECT_ERROR_FMT_LEN         41
#define DIRECT_INC_CANNOT_OPEN       (ag_text_strtable+1567)
#define DIRECT_INC_CANNOT_OPEN_LEN   4
#define DIRECT_INC_CANNOT_READ       (ag_text_strtable+1572)
#define DIRECT_INC_CANNOT_READ_LEN   4
#define DIRECT_INC_DEF_SFX           (ag_text_strtable+1577)
#define DIRECT_INC_DEF_SFX_LEN       3
#define DIRECT_NOENDIF_FMT           (ag_text_strtable+1627)
#define DIRECT_NOENDIF_FMT_LEN       51
#define DIRECT_NOMATCH_FMT           (ag_text_strtable+1679)
#define DIRECT_NOMATCH_FMT_LEN       69
#define DIRECT_SHELL_COMP_DEFS       (ag_text_strtable+1800)
#define DIRECT_SHELL_COMP_DEFS_LEN   20
#define DIRECT_SHELL_END_SHELL       (ag_text_strtable+1821)
#define DIRECT_SHELL_END_SHELL_LEN   10
#define DIRECT_SHELL_NOEND           (ag_text_strtable+1749)
#define DIRECT_SHELL_NOEND_LEN       50
#define DONE_CHECK_DONE              (ag_text_strtable+1832)
#define DONE_CHECK_DONE_LEN          16
#define DONE_CHECK_REDONE            (ag_text_strtable+1849)
#define DONE_CHECK_REDONE_LEN        19
#define DOOPT_TOO_MANY_DEFS          (ag_text_strtable+1878)
#define DOOPT_TOO_MANY_DEFS_LEN      37
#define DOT_SFX_FMT                  (ag_text_strtable+1869)
#define DOT_SFX_FMT_LEN              5
#define DOUBLE_NEWLINE               (ag_text_strtable+1875)
#define DOUBLE_NEWLINE_LEN           2
#define DO_STDOUT_TPL_ABANDONED      (ag_text_strtable+1916)
#define DO_STDOUT_TPL_ABANDONED_LEN  23
#define DO_STDOUT_TPL_BADR           (ag_text_strtable+1940)
#define DO_STDOUT_TPL_BADR_LEN       27
#define DO_STDOUT_TPL_CONTENT        (ag_text_strtable+1968)
#define DO_STDOUT_TPL_CONTENT_LEN    25
#define DO_STDOUT_TPL_NOSFX          (ag_text_strtable+1994)
#define DO_STDOUT_TPL_NOSFX_LEN      8
#define DO_STDOUT_TPL_START_STD      (ag_text_strtable+2003)
#define DO_STDOUT_TPL_START_STD_LEN  25
#define DO_STDOUT_TPL_STDOUT         (ag_text_strtable+2029)
#define DO_STDOUT_TPL_STDOUT_LEN     6
#define DO_SUFFIX_BAD_CHARS          (ag_text_strtable+2036)
#define DO_SUFFIX_BAD_CHARS_LEN      35
#define DO_SUFFIX_EMPTY              (ag_text_strtable+2072)
#define DO_SUFFIX_EMPTY_LEN          19
#define DUP_VALUE_INDEX              (ag_text_strtable+2092)
#define DUP_VALUE_INDEX_LEN          33
#define ELIF_CONTEXT_MSG             (ag_text_strtable+2126)
#define ELIF_CONTEXT_MSG_LEN         36
#define EMIT_INVAL_PORT              (ag_text_strtable+2163)
#define EMIT_INVAL_PORT_LEN          26
#define END_C_COMMENT                (ag_text_strtable+2190)
#define END_C_COMMENT_LEN            2
#define ERROR_IN_TPL_FMT             (ag_text_strtable+2199)
#define ERROR_IN_TPL_FMT_LEN         31
#define ERROR_STR                    (ag_text_strtable+2193)
#define ERROR_STR_LEN                5
#define EVAL_EXPR_BLOCK_IN_EVAL      (ag_text_strtable+2231)
#define EVAL_EXPR_BLOCK_IN_EVAL_LEN  47
#define EVAL_EXPR_PROG_ERR           (ag_text_strtable+2279)
#define EVAL_EXPR_PROG_ERR_LEN       35
#define EVAL_TRUE_FALSE_STR          (ag_text_strtable+2359)
#define EVAL_TRUE_FALSE_STR_LEN      5
#define EXIT_CLEANUP_DONE_FMT        (ag_text_strtable+2315)
#define EXIT_CLEANUP_DONE_FMT_LEN    43
#define EXIT_CLEANUP_MULLIGAN        (ag_text_strtable+2365)
#define EXIT_CLEANUP_MULLIGAN_LEN    21
#define EXIT_CLEANUP_NOWAIT          (ag_text_strtable+2387)
#define EXIT_CLEANUP_NOWAIT_LEN      10
#define EXIT_CLEANUP_STR             (ag_text_strtable+2398)
#define EXIT_CLEANUP_STR_LEN         93
#define EXIT_CLEANUP_WAITED          (ag_text_strtable+2492)
#define EXIT_CLEANUP_WAITED_LEN      6
#define EXP_FMT_BAD_LIC              (ag_text_strtable+2499)
#define EXP_FMT_BAD_LIC_LEN          18
#define EXP_FMT_DNE                  (ag_text_strtable+2518)
#define EXP_FMT_DNE1                 (ag_text_strtable+2659)
#define EXP_FMT_DNE1_LEN             46
#define EXP_FMT_DNE2                 (ag_text_strtable+2706)
#define EXP_FMT_DNE2_LEN             145
#define EXP_FMT_DNE_LEN              140
#define EXTRACT_CAVEAT               (ag_text_strtable+2904)
#define EXTRACT_CAVEAT_LEN           26
#define EXTRACT_END                  (ag_text_strtable+2984)
#define EXTRACT_END_LEN              5
#define EXTRACT_START                (ag_text_strtable+2990)
#define EXTRACT_START_LEN            5
#define FALSE_NAME_STR               (ag_text_strtable+2359)
#define FALSE_NAME_STR_LEN           5
#define FILE_LINE_FMT                (ag_text_strtable+2996)
#define FILE_LINE_FMT_LEN            13
#define FILL_IN_VAL_NO_ASSIGN        (ag_text_strtable+2931)
#define FILL_IN_VAL_NO_ASSIGN_LEN    24
#define FILL_IN_VAL_NO_SEP           (ag_text_strtable+2956)
#define FILL_IN_VAL_NO_SEP_LEN       27
#define FIND_DIRECT_UNKNOWN          (ag_text_strtable+2852)
#define FIND_DIRECT_UNKNOWN_LEN      51
#define FIND_ENT_FAIL                (ag_text_strtable+3010)
#define FIND_ENT_FAIL_LEN            7
#define FIND_ENT_SUCC                (ag_text_strtable+3018)
#define FIND_ENT_SUCC_LEN            8
#define FIND_FILE_CURDIR             (ag_text_strtable+3027)
#define FIND_FILE_CURDIR_LEN         1
#define FIND_FILE_DIR_FMT            (ag_text_strtable+3029)
#define FIND_FILE_DIR_FMT_LEN        5
#define FIND_FILE_NAME               (ag_text_strtable+3035)
#define FIND_FILE_NAME_LEN           9
#define FIND_LIC_TEXT_AGPL           (ag_text_strtable+3045)
#define FIND_LIC_TEXT_AGPL_LEN       4
#define FIND_LIC_TEXT_BAD_FILE       (ag_text_strtable+3050)
#define FIND_LIC_TEXT_BAD_FILE_LEN   14
#define FIND_LIC_TEXT_INVAL          (ag_text_strtable+3065)
#define FIND_LIC_TEXT_INVAL_LEN      24
#define FIND_LIC_TEXT_LGPL           (ag_text_strtable+3090)
#define FIND_LIC_TEXT_LGPL_LEN       4
#define FIND_LIC_TEXT_MBSD           (ag_text_strtable+3095)
#define FIND_LIC_TEXT_MBSD_LEN       4
#define FIND_LIC_TEXT_NO_LIC         (ag_text_strtable+3100)
#define FIND_LIC_TEXT_NO_LIC_LEN     9
#define FIND_LIC_TEXT_OPEN           (ag_text_strtable+1567)
#define FIND_LIC_TEXT_OPEN_LEN       4
#define FIND_LIC_TEXT_SFX            (ag_text_strtable+3110)
#define FIND_LIC_TEXT_SFX_LEN        3
#define FIND_MAC_END_EMPTY           (ag_text_strtable+3150)
#define FIND_MAC_END_EMPTY_LEN       35
#define FIND_MAC_END_NESTED          (ag_text_strtable+3114)
#define FIND_MAC_END_NESTED_LEN      18
#define FIND_MAC_END_NOPE            (ag_text_strtable+3133)
#define FIND_MAC_END_NOPE_LEN        16
#define FN_BOGUS_FMT                 (ag_text_strtable+3217)
#define FN_BOGUS_FMT_LEN             55
#define FN_BOGUS_HUH                 (ag_text_strtable+3273)
#define FN_BOGUS_HUH_LEN             13
#define FN_DEBUG                     (ag_text_strtable+3186)
#define FN_DEBUG_GRAPHIC             (ag_text_strtable+3287)
#define FN_DEBUG_GRAPHIC_LEN         5
#define FN_DEBUG_LEN                 30
#define FN_IF_ELSE                   (ag_text_strtable+3293)
#define FN_IF_ELSE_LEN               11
#define FOR_IN_LISTLESS              (ag_text_strtable+3305)
#define FOR_IN_LISTLESS_LEN          24
#define FOR_ITERATION_FMT            (ag_text_strtable+3337)
#define FOR_ITERATION_FMT_LEN        52
#define FS_ERR_WARNING               (ag_text_strtable+1581)
#define FS_ERR_WARNING_LEN           45
#define GET_NOTHING_FMT              (ag_text_strtable+3390)
#define GET_NOTHING_FMT_LEN          41
#define GOT_NOTHING_FMT              (ag_text_strtable+3432)
#define GOT_NOTHING_FMT_LEN          33
#define GUARD_SFX                    (ag_text_strtable+3330)
#define GUARD_SFX_LEN                6
#define GUILE_CMD_FAIL_FMT           (ag_text_strtable+3466)
#define GUILE_CMD_FAIL_FMT_LEN       73
#define GUILE_VERSION_BAD            (ag_text_strtable+3540)
#define GUILE_VERSION_BAD_LEN        37
#define GUILE_VERSION_WRONG          (ag_text_strtable+3578)
#define GUILE_VERSION_WRONG_LEN      48
#define GUILE_WARN_DEP_STR           (ag_text_strtable+3627)
#define GUILE_WARN_DEP_STR_LEN       21
#define GUILE_WARN_NO_ENV            (ag_text_strtable+3649)
#define GUILE_WARN_NO_ENV_LEN        24
#define HANDLE_EOL__ELS              (ag_text_strtable+3674)
#define HANDLE_EOL__ELS_LEN          3
#define HANDLE_EOL__THE              (ag_text_strtable+3678)
#define HANDLE_EOL__THE_LEN          3
#define HANDLE_HASH_BAD_TPL          (ag_text_strtable+3682)
#define HANDLE_HASH_BAD_TPL_LEN      21
#define HANDLE_HASH_SHELL            (ag_text_strtable+3704)
#define HANDLE_HASH_SHELL_LEN        6
#define HANDLE_SED_ELSE              (ag_text_strtable+3711)
#define HANDLE_SED_ELSE_LEN          3
#define HANDLE_SED_ENDIF             (ag_text_strtable+3715)
#define HANDLE_SED_ENDIF_LEN         4
#define HANDLE_SED_IFDEF             (ag_text_strtable+3720)
#define HANDLE_SED_IFDEF_LEN         5
#define HANDLE_SED_IFNDEF            (ag_text_strtable+3726)
#define HANDLE_SED_IFNDEF_LEN        6
#define HEADER_STR                   (ag_text_strtable+3733)
#define HEADER_STR_LEN               6
#define HERE_ENDLESS_STR             (ag_text_strtable+3740)
#define HERE_ENDLESS_STR_LEN         23
#define HERE_MARK_TOO_LONG           (ag_text_strtable+3824)
#define HERE_MARK_TOO_LONG_LEN       32
#define HERE_MISS_MARK_STR           (ag_text_strtable+3764)
#define HERE_MISS_MARK_STR_LEN       27
#define HIDE_EMAIL_END_FMT           (ag_text_strtable+3857)
#define HIDE_EMAIL_END_FMT_LEN       48
#define HIDE_EMAIL_START_STR         (ag_text_strtable+3906)
#define HIDE_EMAIL_START_STR_LEN     166
#define HTML_DEC_DIGIT               (ag_text_strtable+3792)
#define HTML_DEC_DIGIT_LEN           5
#define INIT_SCM_DEBUG_FMT           (ag_text_strtable+3798)
#define INIT_SCM_DEBUG_FMT_LEN       25
#define INIT_SCM_ERRS_FMT            (ag_text_strtable+4073)
#define INIT_SCM_ERRS_FMT_LEN        125
#define KR_STRING_NEWLINE            (ag_text_strtable+4325)
#define KR_STRING_NEWLINE_LEN        4
#define LC_ALL_IS_C                  (ag_text_strtable+4330)
#define LC_ALL_IS_C_LEN              8
#define LD_BOGUS_UNKNOWN             (ag_text_strtable+4199)
#define LD_BOGUS_UNKNOWN_LEN         53
#define LD_CASE_NO_ESAC              (ag_text_strtable+4339)
#define LD_CASE_NO_ESAC_LEN          14
#define LD_CASE_NO_EXPR              (ag_text_strtable+4354)
#define LD_CASE_NO_EXPR_LEN          19
#define LD_DEF_NEED_NAME             (ag_text_strtable+4374)
#define LD_DEF_NEED_NAME_LEN         22
#define LD_DEF_WOOPS                 (ag_text_strtable+4397)
#define LD_DEF_WOOPS_LEN             24
#define LD_EXPR_BAD_NAME             (ag_text_strtable+4422)
#define LD_EXPR_BAD_NAME_LEN         23
#define LD_EXPR_NEED_TWO             (ag_text_strtable+4446)
#define LD_EXPR_NEED_TWO_LEN         25
#define LD_EXPR_NO_SPACE             (ag_text_strtable+4472)
#define LD_EXPR_NO_SPACE_LEN         28
#define LD_EXPR_NO_TEXT              (ag_text_strtable+4501)
#define LD_EXPR_NO_TEXT_LEN          25
#define LD_FOR_IN                    (ag_text_strtable+4527)
#define LD_FOR_INVALID_VAR           (ag_text_strtable+4530)
#define LD_FOR_INVALID_VAR_LEN       25
#define LD_FOR_IN_LEN                2
#define LD_FOR_NAMELESS              (ag_text_strtable+4253)
#define LD_FOR_NAMELESS_LEN          32
#define LD_FOR_NO_ENDFOR             (ag_text_strtable+4556)
#define LD_FOR_NO_ENDFOR_LEN         16
#define LD_IF_NO_ENDIF               (ag_text_strtable+4573)
#define LD_IF_NO_ENDIF_LEN           15
#define LD_INC_NO_FNAME              (ag_text_strtable+4286)
#define LD_INC_NO_FNAME_LEN          38
#define LD_LIB_PATH_PFX              (ag_text_strtable+4589)
#define LD_LIB_PATH_PFX_LEN          16
#define LD_LIB_PATH_STR              (ag_text_strtable+4606)
#define LD_LIB_PATH_STR_LEN          15
#define LD_SEL_EMPTY                 (ag_text_strtable+4622)
#define LD_SEL_EMPTY_LEN             16
#define LD_SEL_INVAL                 (ag_text_strtable+4639)
#define LD_SEL_INVAL_LEN             24
#define LD_UNKNOWN_INVAL_DEF         (ag_text_strtable+4422)
#define LD_UNKNOWN_INVAL_DEF_LEN     23
#define LD_WHILE_NO_ENDWHILE         (ag_text_strtable+4664)
#define LD_WHILE_NO_ENDWHILE_LEN     18
#define LD_WHILE_NO_EXPR             (ag_text_strtable+4683)
#define LD_WHILE_NO_EXPR_LEN         20
#define LINE_CONCAT3_FMT             (ag_text_strtable+4704)
#define LINE_CONCAT3_FMT_LEN         8
#define LISTEN_PORT_FMT              (ag_text_strtable+4883)
#define LISTEN_PORT_FMT_LEN          31
#define LOAD_CGI_DEFS_MARKER         (ag_text_strtable+4713)
#define LOAD_CGI_DEFS_MARKER_LEN     21
#define LOAD_CGI_GET_NAME            (ag_text_strtable+4735)
#define LOAD_CGI_GET_NAME_LEN        3
#define LOAD_CGI_INVAL_REQ_FMT       (ag_text_strtable+4915)
#define LOAD_CGI_INVAL_REQ_FMT_LEN   34
#define LOAD_CGI_NO_DATA_MSG         (ag_text_strtable+4739)
#define LOAD_CGI_NO_DATA_MSG_LEN     25
#define LOAD_CGI_READ_NAME           (ag_text_strtable+1572)
#define LOAD_CGI_READ_NAME_LEN       4
#define LOAD_CGI_READ_WHAT           (ag_text_strtable+4765)
#define LOAD_CGI_READ_WHAT_LEN       8
#define LOAD_FILE_SHORT_NAME         (ag_text_strtable+4950)
#define LOAD_FILE_SHORT_NAME_LEN     35
#define LOAD_MACS_BAD_PARSE          (ag_text_strtable+4986)
#define LOAD_MACS_BAD_PARSE_LEN      33
#define LOAD_SCM_ENDLESS             (ag_text_strtable+5020)
#define LOAD_SCM_ENDLESS_LEN         40
#define LOAD_TPL_CANNOT_MAP          (ag_text_strtable+4774)
#define LOAD_TPL_CANNOT_MAP_LEN      13
#define LOAD_TPL_CANNOT_OPEN         (ag_text_strtable+4788)
#define LOAD_TPL_CANNOT_OPEN_LEN     28
#define LOAD_TPL_CANNOT_STAT         (ag_text_strtable+3100)
#define LOAD_TPL_CANNOT_STAT_LEN     9
#define LOAD_TPL_IRREGULAR           (ag_text_strtable+4817)
#define LOAD_TPL_IRREGULAR_LEN       16
#define LOAD_TPL_SFX_AGL             (ag_text_strtable+4834)
#define LOAD_TPL_SFX_AGL_LEN         3
#define LOAD_TPL_SFX_TPL             (ag_text_strtable+4838)
#define LOAD_TPL_SFX_TPL_LEN         3
#define LOG_SEP_FMT                  (ag_text_strtable+5061)
#define LOG_SEP_FMT_LEN              31
#define MAKE_GPERF_CLEANUP           (ag_text_strtable+5093)
#define MAKE_GPERF_CLEANUP_LEN       31
#define MAKE_SCRIPT_NL               (ag_text_strtable+4842)
#define MAKE_SCRIPT_NL_LEN           5
#define MALLOC_FAIL_FMT              (ag_text_strtable+5125)
#define MALLOC_FAIL_FMT_LEN          27
#define MEM_FILE_STR                 (ag_text_strtable+4848)
#define MEM_FILE_STR_LEN             11
#define MKSTEMP_FAIL_FMT             (ag_text_strtable+4860)
#define MKSTEMP_FAIL_FMT_LEN         22
#define MK_GPERF_SCRIPT              (ag_text_strtable+5153)
#define MK_GPERF_SCRIPT_LEN          1187
#define MK_HEAD_GUARD_GUARD          (ag_text_strtable+6466)
#define MK_HEAD_GUARD_GUARD_LEN      27
#define MK_HEAD_GUARD_SCM            (ag_text_strtable+6341)
#define MK_HEAD_GUARD_SCM_LEN        52
#define MK_LIC_BIG_OWN               (ag_text_strtable+6522)
#define MK_LIC_BIG_OWN_LEN           12
#define MK_LIC_BIG_PFX               (ag_text_strtable+6535)
#define MK_LIC_BIG_PFX_LEN           13
#define MK_LIC_BIG_PROG              (ag_text_strtable+6549)
#define MK_LIC_BIG_PROG_LEN          19
#define MK_LIC_NOT_STR               (ag_text_strtable+6569)
#define MK_LIC_NOT_STR_LEN           28
#define MK_LIC_NO_LIC                (ag_text_strtable+6598)
#define MK_LIC_NO_LIC_LEN            23
#define MK_LIC_NO_OPEN               (ag_text_strtable+6394)
#define MK_LIC_NO_OPEN_LEN           32
#define MK_LIC_OWN                   (ag_text_strtable+6622)
#define MK_LIC_OWN_LEN               7
#define MK_LIC_PFX                   (ag_text_strtable+6630)
#define MK_LIC_PFX_LEN               5
#define MK_LIC_PROG                  (ag_text_strtable+6636)
#define MK_LIC_PROG_LEN              9
#define MK_LIC_SFX                   (ag_text_strtable+3110)
#define MK_LIC_SFX_LEN               3
#define MK_LIC_TOO_BIG_FMT           (ag_text_strtable+6494)
#define MK_LIC_TOO_BIG_FMT_LEN       27
#define MK_LIC_YRS                   (ag_text_strtable+6646)
#define MK_LIC_YRS_LEN               7
#define MK_TMP_DIR_CMD               (ag_text_strtable+6427)
#define MK_TMP_DIR_CMD_LEN           38
#define NAMED_VALUES_NO_NAME         (ag_text_strtable+6654)
#define NAMED_VALUES_NO_NAME_LEN     17
#define NAMED_VALUES_WHERE           (ag_text_strtable+6831)
#define NAMED_VALUES_WHERE_LEN       27
#define NEWLINE                      (ag_text_strtable+6672)
#define NEWLINE_LEN                  1
#define NEXT_PM_TOKEN_INVALID        (ag_text_strtable+6674)
#define NEXT_PM_TOKEN_INVALID_LEN    25
#define NOT_STR_FMT                  (ag_text_strtable+6700)
#define NOT_STR_FMT_LEN              26
#define NO_IF_EXPR                   (ag_text_strtable+6727)
#define NO_IF_EXPR_LEN               17
#define NO_NAME_STR                  (ag_text_strtable+6745)
#define NO_NAME_STR_LEN              2
#define NO_TEMPLATE_ERR_MSG          (ag_text_strtable+6748)
#define NO_TEMPLATE_ERR_MSG_LEN      25
#define NULL_FILE_NAME_STR           (ag_text_strtable+6774)
#define NULL_FILE_NAME_STR_LEN       14
#define OPEN_OUTPUT_BAD_FMT          (ag_text_strtable+6859)
#define OPEN_OUTPUT_BAD_FMT_LEN      38
#define OPEN_OUTPUT_NULL             (ag_text_strtable+6789)
#define OPEN_OUTPUT_NULL_LEN         4
#define OUTPUT_NO_OPEN               (ag_text_strtable+6794)
#define OUTPUT_NO_OPEN_LEN           15
#define OUTPUT_NO_UNLINK             (ag_text_strtable+6810)
#define OUTPUT_NO_UNLINK_LEN         6
#define OUTPUT_TEMP_FILE             (ag_text_strtable+6817)
#define OUTPUT_TEMP_FILE_LEN         13
#define OUT_ADD_INVALID              (ag_text_strtable+6898)
#define OUT_ADD_INVALID_LEN          34
#define OUT_CLOSE_TRACE_WRAP         (ag_text_strtable+7061)
#define OUT_CLOSE_TRACE_WRAP_LEN     8
#define OUT_PUSH_NEW_FAIL            (ag_text_strtable+7070)
#define OUT_PUSH_NEW_FAILED          (ag_text_strtable+6933)
#define OUT_PUSH_NEW_FAILED_LEN      34
#define OUT_PUSH_NEW_FAIL_LEN        10
#define OUT_RESUME_CANNOT            (ag_text_strtable+6968)
#define OUT_RESUME_CANNOT_LEN        44
#define OUT_SUSPEND_CANNOT           (ag_text_strtable+7013)
#define OUT_SUSPEND_CANNOT_LEN       47
#define OUT_SWITCH_FAIL              (ag_text_strtable+7081)
#define OUT_SWITCH_FAIL_LEN          7
#define PARSE_INPUT_AG_DEF_STR       (ag_text_strtable+7089)
#define PARSE_INPUT_AG_DEF_STR_LEN   25
#define PATH_TOO_BIG                 (ag_text_strtable+7115)
#define PATH_TOO_BIG_LEN             23
#define PIPE_DEFS_STDIN_NAME         (ag_text_strtable+7139)
#define PIPE_DEFS_STDIN_NAME_LEN     5
#define PIPE_DEFS_STDIN_STR          (ag_text_strtable+1404)
#define PIPE_DEFS_STDIN_STR_LEN      1
#define PIPE_DUP2_NAME_STR           (ag_text_strtable+7145)
#define PIPE_DUP2_NAME_STR_LEN       4
#define PIPE_FIFO_IN_NAME_FMT        (ag_text_strtable+7150)
#define PIPE_FIFO_IN_NAME_FMT_LEN    5
#define PIPE_FIFO_OPEN               (ag_text_strtable+1567)
#define PIPE_FIFO_OPEN_LEN           4
#define PIPE_FIFO_OUT_NAME_FMT       (ag_text_strtable+7156)
#define PIPE_FIFO_OUT_NAME_FMT_LEN   6
#define PIPE_FORK_NAME               (ag_text_strtable+7163)
#define PIPE_FORK_NAME_LEN           4
#define PIPE_MKFIFO_NAME             (ag_text_strtable+7168)
#define PIPE_MKFIFO_NAME_LEN         6
#define PIPE_POLL_NAME_STR           (ag_text_strtable+7175)
#define PIPE_POLL_NAME_STR_LEN       4
#define PIPE_WRITE_NAME_STR          (ag_text_strtable+7180)
#define PIPE_WRITE_NAME_STR_LEN      5
#define PORT_NUM_BAD                 (ag_text_strtable+7186)
#define PORT_NUM_BAD_LEN             26
#define PREP_INVOKE_NO_NAME          (ag_text_strtable+7265)
#define PREP_INVOKE_NO_NAME_LEN      32
#define PREP_INVOKE_NO_SEP           (ag_text_strtable+7298)
#define PREP_INVOKE_NO_SEP_LEN       41
#define PRINT_DEF_SHOW_FMT           (ag_text_strtable+7340)
#define PRINT_DEF_SHOW_FMT_LEN       31
#define PRINT_DEF_SPACES             (ag_text_strtable+7372)
#define PRINT_DEF_SPACES_LEN         32
#define PROC_TPL_BOGUS_RET           (ag_text_strtable+1940)
#define PROC_TPL_BOGUS_RET_LEN       27
#define PROC_TPL_START               (ag_text_strtable+7213)
#define PROC_TPL_START_LEN           21
#define PSEUDO_MACRO_NAME_STR        (ag_text_strtable+7235)
#define PSEUDO_MACRO_NAME_STR_LEN    12
#define PSEUDO_MAC_BAD_ENDER         (ag_text_strtable+7405)
#define PSEUDO_MAC_BAD_ENDER_LEN     36
#define PSEUDO_MAC_BAD_FSM           (ag_text_strtable+7248)
#define PSEUDO_MAC_BAD_FSM_LEN       16
#define PSEUDO_MAC_BAD_LENGTH        (ag_text_strtable+7442)
#define PSEUDO_MAC_BAD_LENGTH_LEN    29
#define PSEUDO_MAC_BAD_MODE          (ag_text_strtable+7551)
#define PSEUDO_MAC_BAD_MODE_LEN      24
#define PSEUDO_MAC_BAD_NOAG5         (ag_text_strtable+7576)
#define PSEUDO_MAC_BAD_NOAG5_LEN     20
#define PSEUDO_MAC_BAD_NOEND         (ag_text_strtable+7597)
#define PSEUDO_MAC_BAD_NOEND_LEN     15
#define PSEUDO_MAC_BAD_NOEOL         (ag_text_strtable+7613)
#define PSEUDO_MAC_BAD_NOEOL_LEN     16
#define PSEUDO_MAC_BAD_NOSTART       (ag_text_strtable+7630)
#define PSEUDO_MAC_BAD_NOSTART_LEN   17
#define PSEUDO_MAC_BAD_NOTPL         (ag_text_strtable+7648)
#define PSEUDO_MAC_BAD_NOTPL_LEN     20
#define PSEUDO_MAC_BAD_PSEUDO        (ag_text_strtable+7669)
#define PSEUDO_MAC_BAD_PSEUDO_LEN    23
#define PSEUDO_MAC_BAD_STARTER       (ag_text_strtable+7472)
#define PSEUDO_MAC_BAD_STARTER_LEN   36
#define PSEUDO_MAC_ERR_FMT           (ag_text_strtable+7509)
#define PSEUDO_MAC_ERR_FMT_LEN       41
#define PSEUDO_MAC_MODE_MARK         (ag_text_strtable+7693)
#define PSEUDO_MAC_MODE_MARK_LEN     3
#define PSEUDO_MAC_TPL_FILE          (ag_text_strtable+7697)
#define PSEUDO_MAC_TPL_FILE_LEN      15
#define READY_INPUT_NODEF            (ag_text_strtable+7713)
#define READY_INPUT_NODEF_LEN        20
#define READY_INPUT_NOT_REG          (ag_text_strtable+7734)
#define READY_INPUT_NOT_REG_LEN      21
#define READY_INPUT_STAT             (ag_text_strtable+7756)
#define READY_INPUT_STAT_LEN         4
#define READ_DEF_NO_DEFS             (ag_text_strtable+7761)
#define READ_DEF_NO_DEFS_LEN         28
#define READ_DEF_OPEN                (ag_text_strtable+1567)
#define READ_DEF_OPEN_LEN            4
#define READ_DEF_READ                (ag_text_strtable+1572)
#define READ_DEF_READ_LEN            4
#define REALLOC_FAIL_FMT             (ag_text_strtable+7926)
#define REALLOC_FAIL_FMT_LEN         36
#define REQUEST_METHOD               (ag_text_strtable+7790)
#define REQUEST_METHOD_LEN           14
#define RESOLVE_SCM_EXACT            (ag_text_strtable+7805)
#define RESOLVE_SCM_EXACT_LEN        11
#define RESOLVE_SCM_INEXACT          (ag_text_strtable+7817)
#define RESOLVE_SCM_INEXACT_LEN      13
#define RESOLVE_SCM_LIST             (ag_text_strtable+7831)
#define RESOLVE_SCM_LIST_LEN         10
#define RESOLVE_SCM_PAIR             (ag_text_strtable+7842)
#define RESOLVE_SCM_PAIR_LEN         10
#define RESOLVE_SCM_PROC             (ag_text_strtable+7853)
#define RESOLVE_SCM_PROC_LEN         23
#define RESOLVE_SCM_UNKNOWN          (ag_text_strtable+7877)
#define RESOLVE_SCM_UNKNOWN_LEN      13
#define RESOLVE_SCM_VECTOR           (ag_text_strtable+7891)
#define RESOLVE_SCM_VECTOR_LEN       12
#define RETURN_FROM_NOWHERE          (ag_text_strtable+7904)
#define RETURN_FROM_NOWHERE_LEN      21
#define RM_MISSING_DE                (ag_text_strtable+7963)
#define RM_MISSING_DE_LEN            39
#define RUN_GPERF_CMD                (ag_text_strtable+8003)
#define RUN_GPERF_CMD_LEN            171
#define RUN_PRINTF_HUH               (ag_text_strtable+3273)
#define RUN_PRINTF_HUH_LEN           13
#define SAFE_PRINTF_BAD_ARGS         (ag_text_strtable+8220)
#define SAFE_PRINTF_BAD_ARGS_LEN     19
#define SAFE_PRINTF_BAD_FMT          (ag_text_strtable+8175)
#define SAFE_PRINTF_BAD_FMT_LEN      44
#define SCHEME_EVAL_ERR_FMT          (ag_text_strtable+8268)
#define SCHEME_EVAL_ERR_FMT_LEN      71
#define SCHEME_INIT_DEBUG_1_6        (ag_text_strtable+8240)
#define SCHEME_INIT_DEBUG_1_6_LEN    11
#define SCHEME_INIT_DEBUG_2_0        (ag_text_strtable+8252)
#define SCHEME_INIT_DEBUG_2_0_LEN    15
#define SCHEME_INIT_TEXT             (ag_text_strtable+8340)
#define SCHEME_INIT_TEXT_LEN         3928
#define SCM2ZCHARS_BAD_VAL           (ag_text_strtable+12269)
#define SCM2ZCHARS_BAD_VAL_LEN       49
#define SCM_ERROR_FMT                (ag_text_strtable+12319)
#define SCM_ERROR_FMT_LEN            50
#define SCM_FALSE_STR                (ag_text_strtable+12370)
#define SCM_FALSE_STR_LEN            2
#define SCM_LIST_STR                 (ag_text_strtable+12373)
#define SCM_LIST_STR_LEN             3
#define SCM_OUT_POP_EMPTY            (ag_text_strtable+12431)
#define SCM_OUT_POP_EMPTY_LEN        48
#define SCM_OUT_POP_NO_REREAD        (ag_text_strtable+12377)
#define SCM_OUT_POP_NO_REREAD_LEN    14
#define SCM_PROC_CAST                (ag_text_strtable+12392)
#define SCM_PROC_CAST_LEN            5
#define SCM_TRUE_STR                 (ag_text_strtable+12398)
#define SCM_TRUE_STR_LEN             2
#define SERV_RESTART                 (ag_text_strtable+12480)
#define SERV_RESTART_LEN             40
#define SET_ORIG_DIR_NO_MEM_MSG      (ag_text_strtable+12521)
#define SET_ORIG_DIR_NO_MEM_MSG_LEN  25
#define SET_TMP_DIR_CMD              (ag_text_strtable+12547)
#define SET_TMP_DIR_CMD_LEN          86
#define SET_WRITE_WARN               (ag_text_strtable+12634)
#define SET_WRITE_WARN_LEN           84
#define SFX_FMT                      (ag_text_strtable+12401)
#define SFX_FMT_LEN                  4
#define SHELL_CMD_FMT                (ag_text_strtable+12719)
#define SHELL_CMD_FMT_LEN            28
#define SHELL_DIE_FMT                (ag_text_strtable+12748)
#define SHELL_DIE_FMT_LEN            41
#define SHELL_INIT_STR               (ag_text_strtable+12790)
#define SHELL_INIT_STR_LEN           1253
#define SHELL_LAST_CMD_MSG           (ag_text_strtable+14044)
#define SHELL_LAST_CMD_MSG_LEN       22
#define SHELL_NO_END_MARK_MSG        (ag_text_strtable+14067)
#define SHELL_NO_END_MARK_MSG_LEN    18
#define SHELL_READ_ERR_FMT           (ag_text_strtable+14146)
#define SHELL_READ_ERR_FMT_LEN       41
#define SHELL_RESTART_MSG            (ag_text_strtable+14086)
#define SHELL_RESTART_MSG_LEN        16
#define SHELL_RES_DISCARD_MSG        (ag_text_strtable+14103)
#define SHELL_RES_DISCARD_MSG_LEN    19
#define SHELL_RES_NULL_MSG           (ag_text_strtable+12406)
#define SHELL_RES_NULL_MSG_LEN       24
#define SHELL_SET_PS4_FMT            (ag_text_strtable+14123)
#define SHELL_SET_PS4_FMT_LEN        22
#define SHELL_UNK_LAST_CMD_MSG       (ag_text_strtable+3273)
#define SHELL_UNK_LAST_CMD_MSG_LEN   13
#define SHELL_XTRACE_CMDS            (ag_text_strtable+14188)
#define SHELL_XTRACE_CMDS_LEN        35
#define SH_DONE_MARK                 (ag_text_strtable+14224)
#define SH_DONE_MARK_LEN             31
#define START_DEP_ARG_FMT            (ag_text_strtable+14370)
#define START_DEP_ARG_FMT_LEN        6
#define START_DEP_FILE_FMT           (ag_text_strtable+14256)
#define START_DEP_FILE_FMT_LEN       113
#define START_DEP_FOPEN_MSG          (ag_text_strtable+14377)
#define START_DEP_FOPEN_MSG_LEN      15
#define START_SCHEME_LIST            (ag_text_strtable+14393)
#define START_SCHEME_LIST_LEN        2
#define STDIN_FILE_NAME              (ag_text_strtable+7139)
#define STDIN_FILE_NAME_LEN          5
#define STRDUP_FAIL_FMT              (ag_text_strtable+14396)
#define STRDUP_FAIL_FMT_LEN          32
#define STR_TO_C_MAP_FAIL            (ag_text_strtable+14429)
#define STR_TO_C_MAP_FAIL_LEN        44
#define STR_TO_C_NAME                (ag_text_strtable+14474)
#define STR_TO_C_NAME_LEN            25
#define SYSCALL_NAME                 (ag_text_strtable+14531)
#define SYSCALL_NAME_LEN             7
#define TAB_FILE_LINE_FMT            (ag_text_strtable+14539)
#define TAB_FILE_LINE_FMT_LEN        17
#define TEMP_SUFFIX                  (ag_text_strtable+14557)
#define TEMP_SUFFIX_LEN              9
#define TOO_MANY_TIMEOUTS_MSG        (ag_text_strtable+14500)
#define TOO_MANY_TIMEOUTS_MSG_LEN    30
#define TPL_FILE_LINE_FMT            (ag_text_strtable+1146)
#define TPL_FILE_LINE_FMT_LEN        15
#define TPL_FILE_NEXT_LINE_FMT       (ag_text_strtable+2996)
#define TPL_FILE_NEXT_LINE_FMT_LEN   13
#define TPL_FILE_TPL                 (ag_text_strtable+4838)
#define TPL_FILE_TPL_LEN             3
#define TPL_INVOKED                  (ag_text_strtable+14618)
#define TPL_INVOKED_LEN              39
#define TPL_MARK                     (ag_text_strtable+14567)
#define TPL_MARK_LEN                 8
#define TPL_WARN_FMT                 (ag_text_strtable+14658)
#define TPL_WARN_FMT_LEN             36
#define TRACE_ADD_SRC_FILE_FMT       (ag_text_strtable+14576)
#define TRACE_ADD_SRC_FILE_FMT_LEN   20
#define TRACE_ADD_TARG_FILE_FMT      (ag_text_strtable+14597)
#define TRACE_ADD_TARG_FILE_FMT_LEN  20
#define TRACE_ADD_TO_ENV_FMT         (ag_text_strtable+14695)
#define TRACE_ADD_TO_ENV_FMT_LEN     27
#define TRACE_AG_ARG_FMT             (ag_text_strtable+14822)
#define TRACE_AG_ARG_FMT_LEN         5
#define TRACE_BUILD_DEFS             (ag_text_strtable+14828)
#define TRACE_BUILD_DEFS_LEN         23
#define TRACE_CASE_FAIL              (ag_text_strtable+14723)
#define TRACE_CASE_FAIL_LEN          31
#define TRACE_CASE_MATCHED           (ag_text_strtable+14755)
#define TRACE_CASE_MATCHED_LEN       33
#define TRACE_CASE_NOMATCH           (ag_text_strtable+14789)
#define TRACE_CASE_NOMATCH_LEN       32
#define TRACE_COPY_MARK              (ag_text_strtable+14852)
#define TRACE_COPY_MARK_LEN          19
#define TRACE_DEF_LOAD               (ag_text_strtable+14872)
#define TRACE_DEF_LOAD_LEN           17
#define TRACE_EVAL_STRING            (ag_text_strtable+14929)
#define TRACE_EVAL_STRING_LEN        30
#define TRACE_FIND_ENT               (ag_text_strtable+14890)
#define TRACE_FIND_ENT_LEN           12
#define TRACE_FN_FOR                 (ag_text_strtable+14960)
#define TRACE_FN_FOR_LEN             37
#define TRACE_FN_FOR_REPEAT          (ag_text_strtable+14903)
#define TRACE_FN_FOR_REPEAT_LEN      25
#define TRACE_FN_FOR_SKIP            (ag_text_strtable+14998)
#define TRACE_FN_FOR_SKIP_LEN        42
#define TRACE_FN_IF                  (ag_text_strtable+15041)
#define TRACE_FN_IF_LEN              43
#define TRACE_FN_IF_NOTHING          (ag_text_strtable+15085)
#define TRACE_FN_IF_NOTHING_LEN      33
#define TRACE_FN_INC_LINE            (ag_text_strtable+14539)
#define TRACE_FN_INC_LINE_LEN        17
#define TRACE_FN_INC_TPL             (ag_text_strtable+15119)
#define TRACE_FN_INC_TPL_LEN         21
#define TRACE_FN_REMAPPED            (ag_text_strtable+15167)
#define TRACE_FN_REMAPPED_LEN        40
#define TRACE_FN_REMAP_BASE          (ag_text_strtable+15141)
#define TRACE_FN_REMAP_BASE_LEN      13
#define TRACE_FN_REMAP_EXPR          (ag_text_strtable+15155)
#define TRACE_FN_REMAP_EXPR_LEN      4
#define TRACE_FN_REMAP_INVOKE        (ag_text_strtable+15160)
#define TRACE_FN_REMAP_INVOKE_LEN    6
#define TRACE_FN_WHILE_END           (ag_text_strtable+15208)
#define TRACE_FN_WHILE_END_LEN       30
#define TRACE_FN_WHILE_START         (ag_text_strtable+15239)
#define TRACE_FN_WHILE_START_LEN     41
#define TRACE_FOR_BY_STEP            (ag_text_strtable+15281)
#define TRACE_FOR_BY_STEP_LEN        62
#define TRACE_FOR_STEP_TOO_FAR       (ag_text_strtable+15344)
#define TRACE_FOR_STEP_TOO_FAR_LEN   38
#define TRACE_MACRO_DEF              (ag_text_strtable+15383)
#define TRACE_MACRO_DEF_LEN          26
#define TRACE_MACRO_FMT              (ag_text_strtable+15410)
#define TRACE_MACRO_FMT_LEN          29
#define TRACE_MATCH_VAL              (ag_text_strtable+15559)
#define TRACE_MATCH_VAL_LEN          18
#define TRACE_MOVE_FMT               (ag_text_strtable+15578)
#define TRACE_MOVE_FMT_LEN           12
#define TRACE_OPEN_OUT               (ag_text_strtable+15591)
#define TRACE_OPEN_OUT_LEN           16
#define TRACE_OUT_DELETE             (ag_text_strtable+15608)
#define TRACE_OUT_DELETE_LEN         26
#define TRACE_OUT_PUSH_NEW           (ag_text_strtable+15635)
#define TRACE_OUT_PUSH_NEW_LEN       19
#define TRACE_OUT_SWITCH             (ag_text_strtable+15655)
#define TRACE_OUT_SWITCH_LEN         19
#define TRACE_POP_FMT                (ag_text_strtable+15675)
#define TRACE_POP_FMT_LEN            8
#define TRACE_RESUME                 (ag_text_strtable+15684)
#define TRACE_RESUME_LEN             16
#define TRACE_RM_SRC_FILE_FMT        (ag_text_strtable+15701)
#define TRACE_RM_SRC_FILE_FMT_LEN    23
#define TRACE_RM_TARG_FILE_FMT       (ag_text_strtable+15725)
#define TRACE_RM_TARG_FILE_FMT_LEN   23
#define TRACE_SEL_MATCH_FULL         (ag_text_strtable+15440)
#define TRACE_SEL_MATCH_FULL_LEN     31
#define TRACE_SHELL_FIRST_START      (ag_text_strtable+15749)
#define TRACE_SHELL_FIRST_START_LEN  20
#define TRACE_SHELL_PID_FMT          (ag_text_strtable+15770)
#define TRACE_SHELL_PID_FMT_LEN      23
#define TRACE_SHELL_RESULT_MSG       (ag_text_strtable+15472)
#define TRACE_SHELL_RESULT_MSG_LEN   59
#define TRACE_SHELL_STARTS_FMT       (ag_text_strtable+15794)
#define TRACE_SHELL_STARTS_FMT_LEN   23
#define TRACE_START_FMT              (ag_text_strtable+15818)
#define TRACE_START_FMT_LEN          29
#define TRACE_START_GUILE            (ag_text_strtable+15532)
#define TRACE_START_GUILE_LEN        26
#define TRACE_SUSPEND                (ag_text_strtable+15848)
#define TRACE_SUSPEND_LEN            14
#define TRACE_TRAP_STATE_FMT         (ag_text_strtable+15863)
#define TRACE_TRAP_STATE_FMT_LEN     15
#define TRACE_UNKNOWN_FMT            (ag_text_strtable+15879)
#define TRACE_UNKNOWN_FMT_LEN        16
#define TRACE_VER_CONVERT            (ag_text_strtable+15896)
#define TRACE_VER_CONVERT_LEN        20
#define TRACE_XTRACE_MSG             (ag_text_strtable+15917)
#define TRACE_XTRACE_MSG_LEN         17
#define TRUE_NAME_STR                (ag_text_strtable+15935)
#define TRUE_NAME_STR_LEN            4
#define ULONG_FMT                    (ag_text_strtable+794)
#define ULONG_FMT_LEN                3
#define UNAME_CALL_NAME              (ag_text_strtable+15940)
#define UNAME_CALL_NAME_LEN          8
#define USAGE_INVAL_DEP_OPT_FMT      (ag_text_strtable+15949)
#define USAGE_INVAL_DEP_OPT_FMT_LEN  35
#define USED_DEFINES_FMT             (ag_text_strtable+15985)
#define USED_DEFINES_FMT_LEN         97
#define USED_DEFINES_LINE_FMT        (ag_text_strtable+16083)
#define USED_DEFINES_LINE_FMT_LEN    3
#define WARN_FMT                     (ag_text_strtable+16087)
#define WARN_FMT_LEN                 12
#define WARN_STR                     (ag_text_strtable+16100)
#define WARN_STR_LEN                 7
#define WORD_FORMAT                  (ag_text_strtable+16108)
#define WORD_FORMAT_LEN              6
#define YES_NAME_STR                 (ag_text_strtable+16115)
#define YES_NAME_STR_LEN             3
#define YYLEX_DF_STR                 (ag_text_strtable+16119)
#define YYLEX_DF_STR_LEN             5
#define YYLEX_ERR_FMT                (ag_text_strtable+16125)
#define YYLEX_ERR_FMT_LEN            158
#define YYLEX_TOKEN_STR              (ag_text_strtable+16284)
#define YYLEX_TOKEN_STR_LEN          12
#define YYLEX_UNENDING_QUOTE         (ag_text_strtable+16297)
#define YYLEX_UNENDING_QUOTE_LEN     32
#define zNil                         (ag_text_strtable+16330)
#define zNil_LEN                     0
extern char const ag_text_strtable[16331];

#define SCHEME_INIT_TEXT_LINENO         434

#endif /* STRINGS_AG_TEXT_H_GUARD */
