#undef   PKGDATADIR
#define  PKGDATADIR "/usr/share/autogen"
#undef   LIBDATADIR
#define  LIBDATADIR "/usr/lib/autogen"
static char const make_prog[] = "/usr/bin/make";

#define  DEFINING 1
#include "autoopts/project.h"
#include "autogen.h"
#include "autogen.c"
#include "defParse-fsm.c"
#include "defLex.c"
#include "directive.c"
#include "funcCase.c"
#include "funcDef.c"
#include "funcEval.c"
#include "funcFor.c"
#include "funcIf.c"
#include "functions.c"
#include "expExtract.c"
#include "expFormat.c"
#include "expGperf.c"
#include "expGuile.c"
#include "expMake.c"
#include "expOutput.c"
#include "expPrint.c"
#include "expState.c"
#include "expString.c"
#include "agShell.c"
#include "ag-text.c"
#include "agCgi.c"
#include "agDep.c"
#include "agInit.c"
#include "agUtils.c"
#include "cgi-fsm.c"
#include "defDirect.c"
#include "defFind.c"
#include "defLoad.c"
#include "fmemopen.c"
#include "loadPseudo.c"
#include "opts.c"
#include "scribble.c"
#include "tpLoad.c"
#include "tpParse.c"
#include "tpProcess.c"
