/*  -*- buffer-read-only: t -*- vi: set ro:
 *
 * DO NOT EDIT THIS FILE   (ag-text.c)
 *
 * It has been AutoGen-ed
 * From the definitions    /u/bkorb/tools/ag/autogen-bld/agen5/ag-text.def
 * and the template file   strings
 *
 *  Copyright (C) 2011-2018 Bruce Korb, all rights reserved.
 *  This is free software. It is licensed for use, modification and
 *  redistribution under the terms of the
 *  Modified (3 clause) Berkeley Software Distribution License
 *  <http://www.xfree86.org/3.3.6/COPYRIGHT2.html>
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name ``Bruce Korb'' nor the name of any other
 *     contributor may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  strings IS PROVIDED BY Bruce Korb ``AS IS'' AND ANY EXPRESS
 *  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 *  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL Bruce Korb OR ANY OTHER CONTRIBUTORS
 *  BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 *  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 *  ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
#include "ag-text.h"

char const ag_text_strtable[16331] =
/*     0 */ "ag-text.c\0"
/*    10 */ "The '%d' address family cannot be handled\0"
/*    52 */ "%s__\0"
/*    57 */ "%s=1\0"
/*    62 */ "ABEND-ing in %s state\n\0"
/*    85 */ "processing template %s\n"
            "            on line %d\n"
            "       for function %s (%d)\n\0"
/*   160 */ "ag-fprintf: 'port' is invalid\0"
/*   190 */ "AUTOGEN_MAKE_DEP\0"
/*   207 */ "autogen5\0"
/*   216 */ "AutoGen caught signal %d (%s) before initialization completed\n\0"
/*   279 */ "AutoGen aborting on signal %d (%s) in state %s\n\0"
/*   327 */ "Scheme definition expression does not yield string:\n\0"
/*   380 */ "Scheme Computed Definitions\0"
/*   408 */ "(alist->autogen-def %s)\0"
/*   432 */ "could not allocate for or formatting failed on:\n"
            "%s\0"
/*   483 */ "asprintfv returned 0x%08X\n\0"
/*   510 */ "%s Error: Invalid input char '%c' in %s on line %d\n\0"
/*   562 */ "Cannot %s from outside a loop\0"
/*   592 */ "Could not resolve macro name: '%s'\0"
/*   627 */ "??? indecipherable error message ???\0"
/*   664 */ "Ill formed name '%s' in %s line %d\n\0"
/*   700 */ "Invalid regular expression:  error %d (%s):\n"
            "%s\0"
/*   747 */ "--base-name name is too long\0"
/*   776 */ "** BOGUS **\0"
/*   788 */ "break\0"
/*   794 */ "%lu\0"
/*   798 */ "fserr %d: cannot %s %s:  %s\n\0"
/*   827 */ "duplicate make target\0"
/*   849 */ "unknown dependency type:  %s\0"
/*   878 */ "Content-type: text/plain\n\n"
            "AutoGen form processing error:\n\0"
/*   936 */ "CGI parsing error:  %s\0"
/*   959 */ "/tmp/cgi-stderr-XXXXXX\0"
/*   982 */ "CLOSING SHELL SERVER - command failure:\n"
            "\t%s\n\0"
/*  1027 */ "%s Error:  %s in %s on line %d\n\0"
/*  1059 */ "DEFINITIONS %s in %s line %d for %s:\n"
            "\t%s\n\0"
/*  1101 */ "Bad regular expression\0"
/*  1124 */ "continue\0"
/*  1133 */ "\\n\"\n"
            "       \"\0"
/*  1146 */ "from %s line %d\0"
/*  1162 */ "block\0"
/*  1168 */ "INVALID\0"
/*  1176 */ "text\0"
/*  1181 */ "unknown\0"
/*  1189 */ " \\\n\0"
/*  1193 */ "\n"
            ".PHONY : clean-%1$s\n\n"
            "clean-%1$s :\n"
            "\trm -f %3$s $(%2$s_TList)\n"
            "\t@-touch -t 199912312359 %1$s\n\0"
/*  1285 */ "\n"
            "%2$s : $(%1$s_SList)\n\n"
            "$(%1$s_TList) : %2$s\n"
            "\t@:\n\0"
/*  1334 */ " \\\n"
            "\t%s\0"
/*  1341 */ "DEPENDENCIES_OUTPUT\0"
/*  1361 */ "\n\n"
            "%s_SList =\0"
/*  1374 */ "%s_TList =\0"
/*  1385 */ "/dev/null\0"
/*  1395 */ "baseless\0"
/*  1404 */ "-\0"
/*  1406 */ "=1\0"
/*  1409 */ "$@\0"
/*  1412 */ "$$/../share/autogen\0"
/*  1432 */ "\n"
            "#\0"
/*  1435 */ "#assert yielded \"%s\":\n"
            "\t`%s`\0"
/*  1463 */ "'#%s' directive encountered out of context\n"
            "\tin %s on line %d\n\0"
/*  1525 */ "#error directive -- in %s on line %d\n"
            "\t%s\n\0"
/*  1567 */ "open\0"
/*  1572 */ "read\0"
/*  1577 */ "def\0"
/*  1581 */ "WARNING: fserr %d (%s) performing '%s' on %s\n\0"
/*  1627 */ "Definition error:  in %s line %d, #endif not found\n\0"
/*  1679 */ "Definition error:  in %s line %d, #%s no matching start/if directive\n\0"
/*  1749 */ "Missing #endshell after '#shell' in %s on line %d\n\0"
/*  1800 */ "Computed Definitions\0"
/*  1821 */ "\n"
            "#endshell\0"
/*  1832 */ "done_check done\n\0"
/*  1849 */ "done_check re-done\n\0"
/*  1869 */ "%s.%s\0"
/*  1875 */ "\n\n\0"
/*  1878 */ "%s ERROR:  Too many definition files\n\0"
/*  1916 */ "%soutput was abandoned\n\0"
/*  1940 */ "%sBogus return from setjmp\n\0"
/*  1968 */ "content-type: text/html\n\n\0"
/*  1994 */ "* NONE *\0"
/*  2003 */ "Starting stdout template\n\0"
/*  2029 */ "stdout\0"
/*  2036 */ "invalid chars in suffix format:  %s\0"
/*  2072 */ "Empty suffix format\0"
/*  2092 */ "Duplicate value index for %s:  %d\0"
/*  2126 */ "#elif directive found out of context\0"
/*  2163 */ "invalid emission port: %lu\0"
/*  2190 */ "*/\0"
/*  2193 */ "ERROR\0"
/*  2199 */ "Error in template %s, line %d\n"
            "\t\0"
/*  2231 */ "attempted to use block macro in eval expression\0"
/*  2279 */ "PROGRAM ERROR:  ambiguous expr code\0"
/*  2315 */ "exit_cleanup %s done\n"
            "===AutoGen ends - %u\n\n\0"
/*  2359 */ "false\0"
/*  2365 */ "exit_cleanup re-done\n\0"
/*  2387 */ "no waiting\0"
/*  2398 */ "(if (> (string-length shell-cleanup) 0) (shellf \"( (%s) & >/dev/null 2>&1 )\" shell-cleanup) )\0"
/*  2492 */ "waited\0"
/*  2499 */ "an unknown license\0"
/*  2518 */ "%6$s%1$sDO NOT EDIT THIS FILE   (%2$s)\n"
            "%1$s\n"
            "%1$sIt has been AutoGen-ed%3$s\n"
            "%1$sFrom the definitions    %4$s\n"
            "%1$sand the template file   %5$s\0"
/*  2659 */ "%s -*- buffer-read-only: t -*- vi: set ro:\n"
            "%s\n\0"
/*  2706 */ "%6$s%1$sEDIT THIS FILE WITH CAUTION  (%2$s)\n"
            "%1$s\n"
            "%1$sIt has been AutoGen-ed%3$s\n"
            "%1$sFrom the definitions    %4$s\n"
            "%1$sand the template file   %5$s\0"
/*  2852 */ "WARNING:  in %s on line %d unknown directive:\n"
            "\t#%s\n\0"
/*  2904 */ "DO NOT CHANGE THIS COMMENT\0"
/*  2931 */ "name not followed by '='\0"
/*  2956 */ "no space separating entries\0"
/*  2984 */ "END  \0"
/*  2990 */ "START\0"
/*  2996 */ "# %2$d \"%1$s\"\0"
/*  3010 */ "failed\n\0"
/*  3018 */ "SUCCESS\n\0"
/*  3027 */ ".\0"
/*  3029 */ "%s/%s\0"
/*  3035 */ "find-file\0"
/*  3045 */ "agpl\0"
/*  3050 */ "read full file\0"
/*  3065 */ "invalid license file: %s\0"
/*  3090 */ "lgpl\0"
/*  3095 */ "mbsd\0"
/*  3100 */ "stat file\0"
/*  3110 */ "lic\0"
/*  3114 */ "macros cannot nest\0"
/*  3133 */ "macro has no end\0"
/*  3150 */ "WARNING: empty macro in %s line %d\n\0"
/*  3186 */ "  --  DEBUG %s -- FOR index %d\0"
/*  3217 */ "%d (%s) is an unknown macro function, or has no handler\0"
/*  3273 */ "? Say, what ?\0"
/*  3287 */ " (%c)\0"
/*  3293 */ "ELSE clause\0"
/*  3305 */ "FOR x IN ... has no list\0"
/*  3330 */ "_GUARD\0"
/*  3337 */ "  -  FOR-%s %s[%d] it#%d: Nxt %d, first=%s, last=%s\n\0"
/*  3390 */ "'get' invoked with no name in %s line %u\n\0"
/*  3432 */ "'get %s' retrieved in %s line %u\n\0"
/*  3466 */ "Failing Guile command:  = = = = =\n\n"
            "%s\n\n"
            "=================================\n\0"
/*  3540 */ "Could not decipher Guile version: %s\n\0"
/*  3578 */ "Compiled Guile version does not match: %s vs %s\n\0"
/*  3627 */ "GUILE_WARN_DEPRECATED\0"
/*  3649 */ "GUILE_WARN_DEPRECATED=no\0"
/*  3674 */ "els\0"
/*  3678 */ "the\0"
/*  3682 */ "Invalid template file\0"
/*  3704 */ "SHELL=\0"
/*  3711 */ "lse\0"
/*  3715 */ "ndif\0"
/*  3720 */ "fdef \0"
/*  3726 */ "fndef \0"
/*  3733 */ "HEADER\0"
/*  3740 */ "Unterminated HereString\0"
/*  3764 */ "HereString missing the mark\0"
/*  3792 */ "&#%d;\0"
/*  3798 */ "(debug-enable 'backtrace)\0"
/*  3824 */ "HereString mark 64 or more chars\0"
/*  3857 */ "');\n"
            "document.write('\" >%s</a>');\n"
            "//-->\n"
            "</script>\0"
/*  3906 */ "<script language=\"JavaScript\" type=\"text/javascript\">\n"
            "<!--\n"
            "var one = 'm&#97;';\n"
            "var two = 'i&#108;t';\n"
            "document.write('<a href=\"' + one + two );\n"
            "document.write('&#111;:\0"
/*  4073 */ "(use-modules (%s)) (read-enable 'positions)\n"
            "(add-hook! before-error-hook error-source-line)\n"
            "(use-modules (ice-9 stack-catch))\0"
/*  4199 */ "Unknown macro or invalid context in %s line %d:\n"
            "\t%s%s\0"
/*  4253 */ "FOR macro requires iterator name\0"
/*  4286 */ "The INCLUDE macro requires a file name\0"
/*  4325 */ "\\n\\\n\0"
/*  4330 */ "LC_ALL=C\0"
/*  4339 */ "ESAC not found\0"
/*  4354 */ "expressionless CASE\0"
/*  4374 */ "DEFINE requires a name\0"
/*  4397 */ "parse ended unexpectedly\0"
/*  4422 */ "Invalid definition name\0"
/*  4446 */ "`?' needs two expressions\0"
/*  4472 */ "No space between expressions\0"
/*  4501 */ "No text for unfound value\0"
/*  4527 */ "in\0"
/*  4530 */ "invalid FOR loop variable\0"
/*  4556 */ "ENDFOR not found\0"
/*  4573 */ "ENDIF not found\0"
/*  4589 */ "LD_LIBRARY_PATH=\0"
/*  4606 */ "LD_LIBRARY_PATH\0"
/*  4622 */ "Empty macro text\0"
/*  4639 */ "Invalid selection clause\0"
/*  4664 */ "ENDWHILE not found\0"
/*  4683 */ "expressionless WHILE\0"
/*  4704 */ "%s\n"
            "%s\n"
            "%s\0"
/*  4713 */ "@@ CGI Definitions @@\0"
/*  4735 */ "GET\0"
/*  4739 */ "No CGI data were received\0"
/*  4765 */ "CGI text\0"
/*  4774 */ "map data file\0"
/*  4788 */ "Could not open template '%s'\0"
/*  4817 */ "not regular file\0"
/*  4834 */ "agl\0"
/*  4838 */ "tpl\0"
/*  4842 */ " ; \\\n\0"
/*  4848 */ "in-mem file\0"
/*  4860 */ "mkstemp failed on `%s'\0"
/*  4883 */ "to port %s with %d type address\0"
/*  4915 */ "invalid CGI request method: ''%s''\0"
/*  4950 */ "Cannot expand directory name:  '%s'\0"
/*  4986 */ "Template parse ended unexpectedly\0"
/*  5020 */ "end of Guile/scheme expression not found\0"
/*  5061 */ "\n\n"
            "* * * * LOG ENTRY %d * * * *\n\0"
/*  5093 */ "(add-cleanup \"rm -rf ${gpdir}\")\0"
/*  5125 */ "malloc of %zd bytes failed\n\0"
/*  5153 */ "gperf --version > /dev/null 2>&1 || die 'no gperf program'\n"
            "test -z ${gpdir} && {\n"
            "  gpdir=`mktemp -d ./.gperf.XXXXXX` 2>/dev/null\n"
            "  test -z \"${gpdir}\" && gpdir=.gperf.$$\n"
            "  test -d ${gpdir} || mkdir ${gpdir} || die \"cannot mkdir ${gpdir}\"\n"
            "}\n"
            "cd ${gpdir} || die cannot cd into ${gpdir}\n"
            "gpdir=`pwd`\n"
            "gperf_%2$s=${gpdir}/%2$s\n\n"
            "{\n"
            "  cat << \\_EOF_\n"
            "%%{\n"
            "#include <stdio.h>\n"
            "%%}\n"
            "struct %2$s_index { char const * name; int const idx; };\n"
            "%%%%\n"
            "_EOF_\n\n"
            "  idx=1\n"
            "  while read f\n"
            "  do echo \"${f}, ${idx}\"\n"
            "     idx=`expr ${idx} + 1`\n"
            "  done << \\_EOLIST_\n"
            "%3$s\n"
            "_EOLIST_\n\n"
            "  cat << \\_EOF_\n"
            "%%%%\n"
            "int main(int argc, char ** argv) {\n"
            "    char * pz = argv[1];\n"
            "    struct %2$s_index const * pI = %2$s_find( pz, strlen( pz ));\n"
            "    if (pI == NULL)\n"
            "        return 1;\n"
            "    printf( \"0x%%02X\\n\", pI->idx );\n"
            "    return 0;\n"
            "}\n"
            "_EOF_\n"
            "} > %2$s.gperf\n\n"
            "exec 2> %2$s.log\n"
            "gperf --language=ANSI-C -H %2$s_hash -N %2$s_find \\\n"
            "      -C -E -I -t %2$s.gperf > %2$s-temp.c || \\\n"
            "   die \"gperf failed on ${gpdir}/%2$s.gperf\n"
            "      `cat %2$s.log`\"\n"
            "egrep -v '^[^#].*_inline' %2$s-temp.c > %2$s.c\n"
            "export CFLAGS=\"$CFLAGS -g\"\n"
            "%1$s %2$s 1>&2\n"
            "test $? -eq 0 -a -x ${gperf_%2$s} || \\\n"
            "  die \"could not '%1$s %2$s' gperf program\n"
            "      `cat %2$s.log`\"\n"
            "exec 2>&8\0"
/*  6341 */ "(define header-file \"%s\") (define header-guard \"%s\")\0"
/*  6394 */ "Could not open license file '%s'\0"
/*  6427 */ "mk_tmp_dir ; echo ${tmp_dir}/ag-XXXXXX\0"
/*  6466 */ "#ifndef %1$s\n"
            "#define %1$s 1\0"
/*  6494 */ "%s may not exceed %d chars\n\0"
/*  6522 */ "owner length\0"
/*  6535 */ "prefix length\0"
/*  6549 */ "program name length\0"
/*  6569 */ "license name is not a string\0"
/*  6598 */ "There is no %s license.\0"
/*  6622 */ "<owner>\0"
/*  6630 */ "<PFX>\0"
/*  6636 */ "<program>\0"
/*  6646 */ "<years>\0"
/*  6654 */ "no macro arg name\0"
/*  6672 */ "\n\0"
/*  6674 */ "Invalid template file: %s\0"
/*  6700 */ "ERROR: %s is not a string\n\0"
/*  6727 */ "expressionless IF\0"
/*  6745 */ "no\0"
/*  6748 */ "no template was specified\0"
/*  6774 */ "NULL file name\0"
/*  6789 */ "null\0"
/*  6794 */ "open for output\0"
/*  6810 */ "unlink\0"
/*  6817 */ "* temp file *\0"
/*  6831 */ "On macro argument # %d:\n"
            "%s\n\0"
/*  6859 */ "Cannot format file name:  \"%s\", %s, %s\0"
/*  6898 */ "No output file specified to add to\0"
/*  6933 */ "failed to create temp file from %s\0"
/*  6968 */ "ERROR: no output file was suspended as '%s'\n\0"
/*  7013 */ "ERROR:  Cannot pop output with no output pushed\0"
/*  7061 */ "%s '%s'\n\0"
/*  7070 */ "open 'wb+'\0"
/*  7081 */ "freopen\0"
/*  7089 */ "Autogen Definitions cgi;\n\0"
/*  7115 */ "AF_UNIX path exceeds %d\0"
/*  7139 */ "stdin\0"
/*  7145 */ "dup2\0"
/*  7150 */ "%s-in\0"
/*  7156 */ "%s-out\0"
/*  7163 */ "fork\0"
/*  7168 */ "mkfifo\0"
/*  7175 */ "poll\0"
/*  7180 */ "write\0"
/*  7186 */ "Invalid port number:  '%s'\0"
/*  7213 */ "Starting %s template\n\0"
/*  7235 */ "pseudo-macro\0"
/*  7248 */ "BROKEN FSM STATE\0"
/*  7265 */ "The INVOKE macro requires a name\0"
/*  7298 */ "The INVOKE macro name not space separated\0"
/*  7340 */ "%s%s[%u] (%s) from %s/%d at %p\n\0"
/*  7372 */ "                                \0"
/*  7405 */ "start marker contained in end marker\0"
/*  7442 */ "start/end macro mark too long\0"
/*  7472 */ "end marker contained in start marker\0"
/*  7509 */ "bad template marker in %s on line %d:\n"
            "\t%s\0"
/*  7551 */ "invalid edit mode marker\0"
/*  7576 */ "need autogen5 marker\0"
/*  7597 */ "need end marker\0"
/*  7613 */ "need end of line\0"
/*  7630 */ "need start marker\0"
/*  7648 */ "need template marker\0"
/*  7669 */ "broken pseudo-macro FSM\0"
/*  7693 */ "-*-\0"
/*  7697 */ "*template file*\0"
/*  7713 */ "@@ No-Definitions @@\0"
/*  7734 */ "open non-regular file\0"
/*  7756 */ "stat\0"
/*  7761 */ "No definition data were read\0"
/*  7790 */ "REQUEST_METHOD\0"
/*  7805 */ "** EXACT **\0"
/*  7817 */ "** INEXACT **\0"
/*  7831 */ "** LIST **\0"
/*  7842 */ "** Pair **\0"
/*  7853 */ "** Procedure 0x%08lX **\0"
/*  7877 */ "** UNKNOWN **\0"
/*  7891 */ "** Vector **\0"
/*  7904 */ "RETURN out of context\0"
/*  7926 */ "realloc of %zd bytes at 0x%p failed\n\0"
/*  7963 */ "attempt to remove unrecorded def entry\n\0"
/*  8003 */ "test -n \"${gperf_%1$s}\" || \\\n"
            "    die 'no environment variable \"gperf_%1$s\"'\n"
            "test -x \"${gperf_%1$s}\" || \\\n"
            "    die \"no gperf program named  ${gperf_%1$s}\"\n"
            "${gperf_%1$s} %2$s\0"
/*  8175 */ "%s ERROR:  %s processing printf format:\n"
            "\t%s\n\0"
/*  8220 */ "Bad args to sprintf\0"
/*  8240 */ "ice-9 debug\0"
/*  8252 */ "system vm trace\0"
/*  8268 */ "Scheme evaluation error.  AutoGen ABEND-ing in template\n"
            "\t%s on line %d\n\0"
/*  8340 */ "(use-modules (ice-9 common-list))\n"
            "(define identifier?\n"
            "(lambda (x) (or (string? x) (symbol? x))))\n"
            "(define normalize-identifier\n"
            "(lambda (x)\n"
            "(if (string? x) (string->symbol x) x)))\n"
            "(define coerce->string\n"
            "(lambda (x)\n"
            "(let ((char->string (lambda (x) (make-string 1 x)))\n"
            "(coercable? (lambda (x)\n"
            "(or (string? x) (boolean? x) (char? x)\n"
            "(symbol? x) (list? x) (number? x)))))\n"
            "(if (not (coercable? x))\n"
            "(error \"Wrong type to coerce->string\" x))\n"
            "(cond\n"
            "((string? x)  (string-append\n"
            "(char->string #\\\") x (char->string #\\\")))\n"
            "((boolean? x) (if x \"#t\" \"#f\"))\n"
            "((char? x)    (char->string x))\n"
            "((number? x)  (number->string x))\n"
            "((symbol? x)  (symbol->string x))\n"
            "((list? x)    (if (every coercable? x)\n"
            "(apply string-append (map coerce->string x))))\n"
            "))))\n"
            "(define alist->autogen-def\n"
            "(lambda (lst . recursive)\n"
            "(if (null? recursive) (set! recursive #f)\n"
            "(set! recursive #t))\n"
            "(let ((res (if recursive \"{\\n\" \"\"))\n"
            "(list-nnul? (lambda (x) (and (list? x) (not (null? x))))))\n"
            "(do ((i lst (cdr i)))\n"
            "((null? i) (if recursive\n"
            "(string-append res \"}\")\n"
            "res))\n"
            "(let* ((kvpair (car i))\n"
            "(value (cdr kvpair))\n"
            "(value-is-alist (if (and (list-nnul? value)\n"
            "(list-nnul? (car value))\n"
            "(list-nnul? (caar value))\n"
            "(identifier? (caaar value)))\n"
            "#t #f)))\n"
            "(set! res (string-append res\n"
            "(coerce->string (normalize-identifier (car kvpair)))\n"
            "\" = \"\n"
            "(if value-is-alist\n"
            "(alist->autogen-def (car value) 1)\n"
            "(coerce->string (cdr kvpair)))\n"
            "\";\\n\"\n"
            ")))))))\n"
            "(define shell-cleanup \"\")\n"
            "(define add-cleanup (lambda (t)\n"
            "(set! shell-cleanup (string-append shell-cleanup \"\\n\" t \"\\n\"))))\n"
            "(define tmp-dir \"\")\n"
            "(define header-file     \"\")\n"
            "(define header-guard    \"\")\n"
            "(define autogen-version \"5.18.16\")\n"
            "(define c-file-line-fmt \"#line %2$d \\\"%1$s\\\"\\n\")\n"
            "(define-macro (defined-as predicate symbol)\n"
            "`(and (defined? ',symbol) (,predicate ,symbol)))\n"
            "(catch #t\n"
            "(lambda () (read-enable 'curly-infix))\n"
            "(lambda args #f))\n"
            "(define html-escape-encode (lambda (str)\n"
            "(string-substitute str\n"
            "'(\"&\"      \"<\"     \">\")\n"
            "'(\"&amp;\"  \"&lt;\"  \"&gt;\"))))\n"
            "(define stt-table   (make-hash-table 31))\n"
            "(define stt-curr    stt-table)\n"
            "(define stt-idx-tbl stt-table)\n"
            "(define stt-idx     0)\n"
            "(define insert-suspended (lambda (susp-name) (begin\n"
            "(out-resume susp-name) (out-pop #t))))\n"
            "(define string-table-new (lambda (st-name) (begin\n"
            "(set! stt-curr (make-hash-table 31))\n"
            "(hash-create-handle! stt-table st-name stt-curr)\n"
            "(out-push-new)\n"
            "(out-suspend st-name)\n"
            "(set! stt-idx-tbl (make-hash-table 31))\n"
            "(hash-create-handle! stt-curr \"string-indexes\" stt-idx-tbl)\n"
            "(hash-create-handle! stt-curr \"current-index\"  0)\n"
            "\"\"\n"
            ")))\n"
            "(define string-table-add (lambda (st-name str-val) (begin\n"
            "(set! stt-curr    (hash-ref stt-table   st-name))\n"
            "(set! stt-idx-tbl (hash-ref stt-curr    \"string-indexes\"))\n"
            "(set! stt-idx     (hash-ref stt-idx-tbl str-val))\n"
            "(if (not (number? stt-idx))\n"
            "(begin\n"
            "(set! stt-idx (hash-ref stt-curr \"current-index\"))\n"
            "(ag-fprintf st-name \"/* %5d */ %s \\\"\\\\0\\\"\\n\"\n"
            "stt-idx (c-string str-val))\n"
            "(hash-create-handle! stt-idx-tbl str-val stt-idx)\n"
            "(hash-set! stt-curr \"current-index\"\n"
            "(+ stt-idx (string-length str-val) 1))\n"
            "))\n"
            "stt-idx\n"
            ")))\n"
            "(define string-table-add-ref (lambda (st-name str-val)\n"
            "(string-append st-name \"+\"\n"
            "(number->string (string-table-add st-name str-val)))))\n"
            "(define emit-string-table (lambda (st-name) (begin\n"
            "(set! stt-curr (hash-ref stt-table   st-name))\n"
            "(set! stt-idx  (hash-ref stt-curr \"current-index\"))\n"
            "(ag-fprintf 0 \"\\nstatic char const %s[%d] =\\n\" st-name stt-idx)\n"
            "(out-resume st-name)\n"
            "(emit (shell (string-append\n"
            "\"sed 's/^ /      /\n"
            "$s/\\\" \\\"\\\\\\\\0\\\"/\\\";/\n"
            "s/\\\" \\\"\\\\\\\\0/\\\\\\\\0/\n"
            "' <<\\\\_EOF_\\n\"\n"
            "(out-pop #t)\n"
            "\"_EOF_\")))\n"
            "(emit \"\\n\")\n"
            ")))\n"
            "(define string-table-size (lambda (st-name)\n"
            "(hash-ref (hash-ref stt-table st-name) \"current-index\")))\n"
            "(define gperf-code (lambda (gp-name) (shellf\n"
            "\"sed -e '1,/^#line/d' \\\n"
            "-e '/#include/d' \\\n"
            "-e '/#line/d' \\\n"
            "-e '/^[ \\t]*$/d' \\\n"
            "-e 's/^const struct /static const struct /' \\\n"
            "-e '/^int main(/,$d' ${gpdir}/%s.c\"\n"
            "gp-name\n"
            ")))\n"
            "(define stack-join (lambda (j-str ag-name)\n"
            "(join j-str (stack ag-name))))\n"
            "(version)\0"
/* 12269 */ "scm_string_length returned wrong value: %d != %d\n\0"
/* 12319 */ "\n"
            "Guile/Scheme evaluation error in %s line %d:  %s\n\0"
/* 12370 */ "#f\0"
/* 12373 */ "...\0"
/* 12377 */ "re-read output\0"
/* 12392 */ "(*)()\0"
/* 12398 */ "#t\0"
/* 12401 */ "%s%s\0"
/* 12406 */ "shell_cmd result is NULL\0"
/* 12431 */ "ERROR:  Cannot pop output with no output pushed\n\0"
/* 12480 */ "SERVER RESTART:  serv_id is NULLPROCESS\n\0"
/* 12521 */ "cannot allocate path name\0"
/* 12547 */ "(set! tmp-dir \"%1$s\")\n"
            "(add-cleanup \"test \\\"${VERBOSE:-false}\\\" = true || rm -rf %1$s\")\0"
/* 12634 */ "Warning: (set-writable) function in %s on line %d:\n"
            "\toverridden by invocation option\n\0"
/* 12719 */ "cd %s\n"
            "%s\n\n"
            "echo\n"
            "echo %s - %d\n\0"
/* 12748 */ "Closing server:  %s signal (%d) received\n\0"
/* 12790 */ "exec 8>&2 2>/dev/null\n\n"
            "if test -n \"${ZSH_VERSION+set}\" && (emulate sh) 1>&2\n"
            "then\n"
            "  emulate sh\n"
            "  NULLCMD=:\n\n"
            "else case `set -o` in *posix*) set -o posix ;; esac\n"
            "fi\n\n"
            "trap_exit() {\n"
            "    case \"$1\" in\n"
            "    0 | 10 | 15 )\n"
            "        exec 1>&- 2>&-\n"
            "        test -d \"${tmp_dir}\" && rm -rf \"${tmp_dir}\"\n"
            "        ;;\n\n"
            "    * )\n"
            "        exec 1>&8\n"
            "        echo \"trapped on signal ${1}\"\n"
            "        test -d \"${tmp_dir}\" && \\\n"
            "            echo \"temp directory has been retained:  ${tmp_dir}\"\n"
            "    esac\n"
            "}\n\n"
            "die() {\n"
            "  echo \"Killing AutoGen ${AG_pid}\"\n"
            "  echo \"FAILURE REASON:  $*\"\n"
            "  kill -15 ${AG_pid}\n"
            "  sleep 1\n"
            "  kill -1  ${AG_pid}\n"
            "  sleep 1\n"
            "  kill -2  ${AG_pid}\n"
            "  sleep 1\n"
            "  kill -9  ${AG_pid}\n"
            "  exit 1\n"
            "} >&8\n\n"
            "mk_tmp_dir() {\n"
            "  test -d \"${tmp_dir}\" && return 0\n"
            "  tmp_dir=`\n"
            "    t=\\`mktemp -d ${TMPDIR}/.ag-XXXXXX\\`\n"
            "    test -d \"${t}\" || {\n"
            "      t=${TMPDIR}/.ag-$$\n"
            "      rm -rf ${t}\n"
            "      mkdir ${t} || die cannot mkdir ${t}\n"
            "    }\n"
            "    chmod 700 ${t} || die cannot chmod 700 ${t}\n"
            "    echo ${t}\n"
            "    ` 2>/dev/null\n"
            "}\n\n"
            "for f in 0 1 2 5 6 7 10 13 14 15\n"
            "do trap \"trap_exit ${f}\" $f ; done\n"
            "test -n \"${CDPATH}\" && {\n"
            "  CDPATH=''\n"
            "  unset CDPATH\n"
            "}\n"
            "( unalias cd ) && unalias cd\n"
            "( set +v     ) && set +v\n"
            "test -z \"${TMPDIR}\" && TMPDIR=/tmp\n"
            "tmp_dir=''\n"
            "export AG_pid TMPDIR\n"
            "AG_pid=\"%u\"\n"
            "AGexe='%s'\n\n"
            "exec 2>&8\0"
/* 14044 */ "\n"
            "Last command issued:\n\0"
/* 14067 */ "feof on data load\n\0"
/* 14086 */ "\n"
            "Server Restart\n\0"
/* 14103 */ "(result discarded)\n\0"
/* 14123 */ "PS4=>${FUNCNAME:-ag}> \0"
/* 14146 */ "fs err %d (%s) reading from server shell\n\0"
/* 14188 */ "set -x\n"
            "trap\n"
            "echo server setup done\n\0"
/* 14224 */ "ShElL-OuTpUt-HaS-bEeN-cOmPlEtEd\0"
/* 14256 */ "# Makefile dependency file created by:\t\t-*- Mode: Makefile -*-\n"
            "# %s\n"
            "# with the following command line arguments:\n\0"
/* 14370 */ "#   %s\0"
/* 14377 */ "fopen for write\0"
/* 14393 */ "'(\0"
/* 14396 */ "strdup of %d byte string failed\n\0"
/* 14429 */ "cannot map unprintable chars to C name chars\0"
/* 14474 */ "ag_scm_string_to_c_name_x\0"
/* 14500 */ "Server shell timed out 5 times\0"
/* 14531 */ "syscall\0"
/* 14539 */ "\tfrom %s line %d\n\0"
/* 14557 */ ".d-XXXXXX\0"
/* 14567 */ "template\0"
/* 14576 */ "add source dep:  %s\n\0"
/* 14597 */ "add target dep:  %s\n\0"
/* 14618 */ "Template macro %s invoked with %d args\n\0"
/* 14658 */ "Warning in template %s, line %d\n"
            "\t%s\n\0"
/* 14695 */ "Adding '%s' to environment\n\0"
/* 14723 */ "CASE string `%s' did not match\n\0"
/* 14755 */ "CASE string `%s' %s matched `%s'\n\0"
/* 14789 */ "CASE no match: `%s' %s vs. `%s'\n\0"
/* 14822 */ " '%s'\0"
/* 14828 */ "eval for arg %d:\n"
            "\t`%s'\n\0"
/* 14852 */ "marker '%s' loaded\n\0"
/* 14872 */ "Definition Load:\n\0"
/* 14890 */ " in \"%s\" -- \0"
/* 14903 */ "FOR %s repeated %d times\n\0"
/* 14929 */ "eval from file %s line %d:\n"
            "%s\n\0"
/* 14960 */ "FOR %s loop in %s on line %d begins:\n\0"
/* 14998 */ "FOR loop skipped - no definition for `%s'\n\0"
/* 15041 */ "IF expression `%s' on line %d yielded true\n\0"
/* 15085 */ "IF `%s' macro selected no clause\n\0"
/* 15119 */ "Template %s included\n\0"
/* 15141 */ "\tbased on %s\n\0"
/* 15155 */ "Expr\0"
/* 15160 */ "Invoke\0"
/* 15167 */ "remapped to '%s' (%2X) in %s at line %d\n\0"
/* 15208 */ "WHILE macro repeated %d times\n\0"
/* 15239 */ "WHILE `%s' loop in %s on line %d begins:\n\0"
/* 15281 */ "\texiting FOR %s from %d to %d by %d:\n"
            "\tmore than %d iterations\n\0"
/* 15344 */ "too many FOR iterations in %s line %d\n\0"
/* 15383 */ "Defining macro %s from %s\n\0"
/* 15410 */ "%-10s (%2X) in %s at line %d\n\0"
/* 15440 */ "Compiling '%s' with bits 0x%lX\n\0"
/* 15472 */ "\n"
            "= = = RESULT %d bytes:\n"
            "%s%s\n"
            "= = = = = = = = = = = = = = =\n\0"
/* 15532 */ "\n"
            "Guile Library Version %s\n\0"
/* 15559 */ "searching for `%s'\0"
/* 15578 */ "%s %s to %s\n\0"
/* 15591 */ "%s '%s' mode %s\n\0"
/* 15608 */ "NOTE:  skipping file '%s'\n\0"
/* 15635 */ "%s -- temp file %s\n\0"
/* 15655 */ "%s from %s to '%s'\n\0"
/* 15675 */ "%s %s%s\n\0"
/* 15684 */ "%s %s from '%s'\n\0"
/* 15701 */ "remove source dep:  %s\n\0"
/* 15725 */ "remove target dep:  %s\n\0"
/* 15749 */ "\n"
            "Server First Start\n\0"
/* 15770 */ "Server shell is pid %u\n\0"
/* 15794 */ "Server shell %s starts\n\0"
/* 15818 */ "\n\n"
            "===AutoGen starts - %u:  %s\0"
/* 15848 */ "%s %s as '%s'\n\0"
/* 15863 */ "Trap state:\n"
            "%s\n\0"
/* 15879 */ "\tcode %lX -- %s\n\0"
/* 15896 */ "0x%016llX <<== '%s'\n\0"
/* 15917 */ "Server traps set\n\0"
/* 15935 */ "true\0"
/* 15940 */ "uname(2)\0"
/* 15949 */ "invalid make dependency option:  %s\0"
/* 15985 */ "echo 'definition names looked up'\n"
            "sed 's/[-^]/-/g'|sort -u -f\n"
            "echo 'end of looked up definitions'\0"
/* 16083 */ "%s\n\0"
/* 16087 */ "WARNING: %s\n\0"
/* 16100 */ "Warning\0"
/* 16108 */ "format\0"
/* 16115 */ "yes\0"
/* 16119 */ "'%s'\n\0"
/* 16125 */ "%s:  in %s on line %d\n"
            "\ttoken in error:  %s\n"
            "\t[[...<error-text>]] %s\n\n"
            "Likely causes:  a mismatched quote, a value that needs quoting,\n"
            "\t\tor a missing semi-colon\n\0"
/* 16284 */ "%s:  ''%s''\n\0"
/* 16297 */ "unterminated quote in definition\0"
/* 16330 */ "";

/* end of ag-text.c */
