#undef   PKGDATADIR
#define  PKGDATADIR "/usr/share/autogen"

#define  DEFINING 1
#include "autoopts/project.h"
#include "opts.c"
#include "getdefs.h"
#include "proto.h"
#include "gdemit.c"
#include "gdinit.c"
#include "getdefs.c"
